#!/usr/bin/python
# -*- coding: utf-8 -*-

# Cagou: desktop/mobile frontend for Salut à Toi XMPP client
# Copyright (C) 2016 Jérôme Poisson (goffi@goffi.org)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from sat.core import log as logging
log = logging.getLogger(__name__)
from sat.core.i18n import _
from sat_frontends.quick_frontend.quick_contact_list import QuickContactList
from sat_frontends.tools import jid
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.listview import ListView
from kivy.adapters.listadapter import ListAdapter
from kivy import properties
from cagou.core import cagou_widget
from cagou.core import image
from cagou import G


PLUGIN_INFO = {
    "name": _(u"contacts"),
    "main": "ContactList",
    "description": _(u"list of contacts"),
    "icon_small": u"{media}/icons/muchoslava/png/contact_list_new_32.png",
    "icon_medium": u"{media}/icons/muchoslava/png/contact_list_new_44.png"
}


class Avatar(image.Image):
    pass


class ContactItem(BoxLayout):
    data = properties.DictProperty()
    jid = properties.StringProperty('')

    def __init__(self, **kwargs):
        super(ContactItem, self).__init__(**kwargs)

    def on_touch_down(self, touch):
        if self.collide_point(*touch.pos):
            # XXX: for now clicking on an item launch the corresponding Chat widget
            #      behaviour should change in the future
            try:
                # FIXME: Q&D way to get chat plugin, should be replaced by a clean method
                #        in host
                plg_infos = [p for p in G.host.getPluggedWidgets() if 'chat' in p['import_name']][0]
            except IndexError:
                log.warning(u"No plugin widget found to display chat")
            else:
                factory = plg_infos['factory']
                G.host.switchWidget(self, factory(plg_infos, jid.JID(self.jid), profiles=iter(G.host.profiles)))


class ContactListView(ListView):
    pass


class ContactList(QuickContactList, cagou_widget.CagouWidget):

    def __init__(self, host, target, profiles):
        QuickContactList.__init__(self, G.host, profiles)
        cagou_widget.CagouWidget.__init__(self)
        self.adapter = ListAdapter(data={},
                                   cls=ContactItem,
                                   args_converter=self.contactDataConverter,
                                   selection_mode='multiple',
                                   allow_empty_selection=True,
                                  )
        self.add_widget(ContactListView(adapter=self.adapter))
        self.postInit()
        self.update()

    def contactDataConverter(self, idx, bare_jid):
        return {"jid": bare_jid, "data": self._items_cache[bare_jid]}

    def update(self, entities=None, type_=None, profile=None):
        log.debug("update: %s %s %s" % (entities, type_, profile))
        # FIXME: for now we update on each event
        # if entities is None and type_ is None:
        self._items_cache = self.items_sorted
        self.adapter.data = self.items_sorted.keys()

